# File rvsao/Emvel/emrslts.x
# September 25, 1997
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1997 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.

#  Log radial velocity from emission line shift

include <imhdr.h>
include "rvsao.h"
include "emv.h"
define	SZ_MONTH	3
define	SZ_YEAR		36

procedure emrslts (specfile, mspec, image, rmode)

char	specfile[ARB]	# Name of spectrum file
int	mspec		# Spectrum number in multispec file (else 0)
pointer	image		# Header structure for spectrum
int	rmode		# Report format (1=normal,2=one-line

int	i, j
char	wch		# Line weight indicator
double	emz		# z from emission lines
double	xcz		# z from cross-correlation
double	spz		# z from cross-correlation and emission
double	spexvel		# Emission line template cross-correlation velocity
double	spexerr		# Emission line template cross-correlation error
int	iczexvel	# Emission line template cross-correlation velocity
int	iczexerr	# Emission line template cross-correlation error
double	spexr		# Emission line template cross-correlation R-value
char	instrument[8]

bool	verbose, clgetb()
double	ra, dec, ut, dj, hdj, velline[MAXREF], errline[MAXREF], eqwline[MAXREF]
double	htline[MAXREF], widline[MAXREF]
double	twt, wt, exptime, alt, az, ha, lat, sha,cha,slat,clat,sdec,cdec,epoch
double	ac,dpi,tpi,rcon, rdec, rha, dindef, dwdp, wl1
int	mm,dd,yyyy, itemp
char	month[SZ_YEAR]
int	icz,iczxc,iczem,icze,iczxce,iczeme
int	strcmp(), strlen()
char	ssvcor[16]
double	wcs_p2w()

include "emv.com"
include "rvsao.com"
include "results.com"

begin
	dindef = INDEFD
	verbose = clgetb ("verbose")
        if (!verbose) return
	call strcpy ("JanFebMarAprMayJunJulAugSepOctNovDec",month, SZ_YEAR)

	call juldate (image, ut, dj, hdj, debug)

	mm = INDEFI
	dd = INDEFI
	yyyy = INDEFI
	call imgdate (image, "DATE-OBS", mm, dd, yyyy)

	if (spvel != dindef) {
	    icz = idnint (spvel)
	    icze = idnint (sperr)
	    }
	else {
	    icz = 0
	    icze = 0
	    }

	if (spxvel != dindef) {
	    iczxc = idnint (spxvel)
	    iczxce = idnint (spxerr)
	    }
	else {
	    iczxc = 0
	    iczxce = 0
	    }

	if (spevel != dindef) {
	    iczem = idnint (spevel)
	    iczeme = idnint (speerr)
	    }
	else {
	    iczem = 0
	    iczeme = 0
	    }

# Set emission line template velocity, if there is one
	spexvel = 0.0
	spexerr = 0.0
	spexr = 0.0
	iczexvel = 0
	iczexerr = 0
	if (strlen (cortemp) > 0 && ntemp > 0) {
	    do itemp = 1, ntemp {
		if (strcmp (cortemp,tempid[1,itemp]) == 0) {
		    spexvel = zvel[itemp]
		    spexerr = czerr[itemp]
		    iczexvel = int (zvel[itemp])
		    iczexerr = int (czerr[itemp])
		    spexr = czr[itemp]
		    }
		}
	    }
	else {
	    spexvel = spxvel
	    spexerr = spxerr
	    if (spxvel != dindef) {
		iczexvel = int (spxvel)
		iczexerr = int (spxerr)
		}
	    else {
		iczexvel = 0
		iczexerr = 0
		}
	    spexr = spxr
	    }

# Set up spectrum number in specid
	if (mspec > 0) {
	    call sprintf (specid,SZ_PATHNAME,"%s[%d]")
		call pargstr (specfile)
		call pargi (mspec)
	    }
	else
	    call strcpy (specfile,specid,SZ_PATHNAME)

	if (rmode == 1) {

#	Read position of object from spectrum header
	    ra = dindef
	    call imgdpar (image,"RA",ra)
	    dec = dindef
	    call imgdpar (image,"DEC",dec)
	    epoch = 1950.d0
	    call imgdpar (image,"EPOCH",epoch)

	    call printf ("\n")
	    call flush (STDOUT)

#	Set type and source of velocity correction to center of solar system
	    switch (svcor) {
		case NONE:
		    call strcpy ("BCV",ssvcor)
		case HCV:
		    call strcpy ("HCV",ssvcor)
		case BCV:
		    call strcpy ("BCV",ssvcor)
		case FHCV:
		    call strcpy ("File HCV",ssvcor)
		case FBCV:
		    if (specvb) 
			call strcpy ("File BCV",ssvcor)
		    else
			call strcpy ("File HCV",ssvcor)
		default:
		    call strcpy ("BCV",ssvcor)
		}

#	Log object information
	    do j = 1, nlogfd {
		call fprintf(logfd[j], "File: %s\n")
		    call pargstr (specid)
		if (ra != dindef && dec != dindef) {
		    call fprintf(logfd[j], "Object: %s  RA: %011.2h Dec: %010.1h %.1f\n")
			call pargstr (specname)
			call pargd (ra)
			call pargd (dec)
			call pargd (epoch)
		    }
		else {
		    call fprintf(logfd[j], "Object: %s  (No position)\n")
			call pargstr (specname)
		    }
	if (debug) {
	    call flush (STDOUT)
	    }
		if (dd != INDEFI && mm != INDEFI && yyyy != INDEFI) {
		    call fprintf(logfd[j], "Observed %2d-%3.3s-%4d %011.2h = JD %.4f %s: %6.2f\n")
			call pargi (dd)
			call pargstr (month[(mm - 1) * SZ_MONTH + 1])
			call pargi (yyyy)
			call pargd (ut)
			call pargd (dj)
			call pargstr (ssvcor)
			call pargd (spechcv)
		    }
		else {
		    call fprintf(logfd[j], "(No observation date) %s: %6.2f\n")
			call pargstr (ssvcor)
			call pargd (spechcv)
		    }
		call flush (logfd[j])

	#  Print combined velocity
		if (spvel != dindef) {
		    spz = spvel / c0
		    call fprintf(logfd[j],
		    "Combined vel = %8.2f +- %7.2f km/sec, z= %6.4f \n")
			call pargd (spvel)
			call pargd (sperr)
			call pargd (spz)
		    call flush (logfd[j])
		    }

	# Print cross-correlation velocity
		if (spxvel != dindef) {
		    xcz = spxvel / c0
		    call fprintf(logfd[j],
		    "Correlation vel = %8.2f +- %7.2f km/sec, z= %6.4f R= %6.1f \n")
			call pargd (spxvel)
			call pargd (spxerr)
			call pargd (xcz)
			call pargd (spxr)
		    call flush (logfd[j])
		    }

	# Print emission line velocity
		if (spevel != dindef) {
		    emz = spevel / c0
		    call fprintf(logfd[j],
		    "Emission vel = %8.2f +- %7.2f km/sec, z= %6.4f for %d/%d lines\n")
			call pargd (spevel)
			call pargd (speerr)
			call pargd (emz)
			call pargi (nfit)
			call pargi (nfound)
		    call flush (logfd[j])
		    }

	# Print emission line information if any were found
		if (nfound > 0) {
		    twt = 0.d0
		    do i = 1, nfound {
			wt = emparams[10,1,i]
			if (wt != 0.d0)
			    twt = twt + 1.d0 / (wt * wt)
			}
		    call fprintf(logfd[j], "\nLine  Rest lam  Obs. lam    ")
		    call fprintf(logfd[j], "Pixel     z         vel    dvel")
		    call fprintf(logfd[j], "    eqw    wt\n")
		    do i = 1, nfound {
			wtobs[i] = emparams[10,1,i]
			if (wtobs[i] > 0) {
			    if (override[i] < 0)
				wch = '-'
			    else if (override[i] > 0)
				wch = '+'
			    else
				wch = ' '
			    }
			else
			    wch = 'X'
			call fprintf(logfd[j],"%4s   %7.2f   %7.2f  %7.2f")
			    call pargstr (nmobs[1,i])
			    call pargd (wlrest[i])
			    call pargd (wlobs[i])
			    call pargd (emparams[4,1,i])
			wt = emparams[10,1,i]
			if (wt != 0.d0)
			    wt = (1.d0 / (wt * wt)) / twt
			call fprintf(logfd[j],"  %6.4f  %8.2f %7.2f %6.2f %6.3f %s\n")
			    call pargd (emparams[9,1,i])
			    call pargd ((c0*emparams[9,1,i]) + spechcv)
			    call pargd (c0*emparams[9,2,i])
			    call pargd (emparams[7,1,i])
			    call pargd (wt)
			    call pargstr (wch)
			}
		    }
		else {
		    call fprintf(logfd[j], "No emission lines found\n")
		    }
		}
	    }

# Single line report
	else if (rmode == 2 || rmode == 5) {
	    call imgspar (image,"INSTRUME",instrument,8)
	    do j = 1, nlogfd {
		call fprintf(logfd[j],"%6s %8s %16s %15.5f %5d %3d %5d %3d %5.1f %5d %3d E %2d %2d")
		    call pargstr (specid)
		    call pargstr (instrument)
		    call pargstr (IM_TITLE(image))
		    call pargd (dj)
		    call pargi (iczem)
		    call pargi (iczeme)
		    if (rmode == 2) {
			call pargi (iczxc)
			call pargi (iczxce)
			call pargd (spxr)
			}
		    else {
			call pargi (iczexvel)
			call pargi (iczexerr)
			call pargd (spexr)
			}
		    call pargi (icz)
		    call pargi (iczxce)
		    call pargi (nfound)
		    call pargi (nfit)
		if (nfit > 0) {
		    do i = 1, nfound {
			wtobs[i] = emparams[10,1,i]
			if (wtobs[i] > 0 && override[i] >= 0) {
			    call fprintf(logfd[j], " %s(%7.2f)")
				call pargstr (nmobs[1,i])
				call pargd (wlrest[i])
			    }
			}
		    }
		call fprintf(logfd[j], "\n")
		}
	    }

# Single line report with one velocity per possible line
	else if (rmode == 3 || rmode == 6) {
	    call imgspar (image,"INSTRUME",instrument,8)
	    call aclrd (velline,MAXREF)

#	Compute individual line velocities from shift
	    if (nfound > 0) {
		do i = 1, nfound {
		    wtobs[i] = emparams[10,1,i]
		    if (wtobs[i] > 0 && override[i] >= 0) {
			do j = 1, nref {
			    if (wlref[j] == wlrest[i])
				velline[j] = (c0 * emparams[9,1,i]) + spechcv
			    }
			}
		    }
		}

	# Print one-line report
	    do j = 1, nlogfd {
		call fprintf(logfd[j],"%6s %8s %16s %15.5f %8.2f %6.2f %8.2f %6.2f %5.1f %8.2f %6.2f E %2d %2d")
		    call pargstr (specid)
		    call pargstr (instrument)
		    call pargstr (IM_TITLE(image))
		    call pargd (dj)
		    call pargd (spevel)
		    call pargd (speerr)
		    if (rmode == 3) {
			call pargd (spxvel)
			call pargd (spxerr)
			call pargd (spxr)
			}
		    else {
			call pargd (spexvel)
			call pargd (spexerr)
			call pargd (spexr)
			}
		    call pargd (spvel)
		    call pargd (sperr)
		    call pargi (spnl)
		    call pargi (spnlf)
		do i = 1, nref {
		    call fprintf (logfd[j]," %8.2f")
			call pargd (velline[i])
		    }
		call fprintf(logfd[j], "\n")
		}
	    }

# Single line report with velocity, error, height, width, and equivalent width
# for each emission line
	else if (rmode == 4) {
	    call imgspar (image,"INSTRUME",instrument,8)
	    call aclrd (velline,MAXREF)
	    call aclrd (errline,MAXREF)
	    call aclrd (htline,MAXREF)
	    call aclrd (widline,MAXREF)
	    call aclrd (eqwline,MAXREF)

#	Compute individual line velocities from shift
	    if (nfound > 0) {
		do i = 1, nfound {
		    wtobs[i] = emparams[10,1,i]
		    if (wtobs[i] > 0 && override[i] >= 0) {
			do j = 1, nref {
			    if (wlref[j] == wlrest[i]) {
				velline[j] = (c0 * emparams[9,1,i]) + spechcv
				errline[j] = c0 * emparams[9,2,i]
				htline[j] = emparams[5,1,i]
				wl1 = wcs_p2w (emparams[4,1,i] - 1.d0)
				dwdp = wlobs[i] - wl1
				widline[j] = emparams[6,1,i] * dwdp
				eqwline[j] = emparams[7,1,i]
				}
			    }
			}
		    }
		}

	# Get sky position
	    ra = 99.d0
	    call imgdpar (image,"RA",ra)
	    dec = 99.d0
	    call imgdpar (image,"DEC",dec)
	    ha = 99.d0
	    call imgdpar (image,"HA",ha)
	    lat = 99.d0
	    call imgdpar (image,"SITELAT",lat)

	    if (debug) {
		call printf ("EMRSLTS: RA: %011.2h Dec: %010.1h HA: %011.2h Lat: %010.1h\n")
		    call pargd (ra)
		    call pargd (dec)
		    call pargd (ha)
		    call pargd (lat)
		}

	# Compute altitude and azimuth
	    if (ha < 12.d0 && ha > -12.d0 && dec <= 90.d0 && dec >= -90.d0 &&
 		lat >= -90.d0 && lat <= 90.d0) {
		dpi = 3.1415926535897932d0
		tpi = 6.283185307179588d0
		rcon = 1.74532925199433d-2
		rha = ha * 15.d0 * rcon
		if (rha == 0)
		    rha = 1.d-5
		sha = sin (rha)
		cha = cos (rha)
		rdec = dec * rcon
		sdec = sin (rdec)
		cdec = cos (rdec)
		slat = sin (lat)
		clat = cos (lat)
		ac = (clat * sdec) - (slat * cdec * cha)
		az = atan (-cdec * sha / abs (ac))
		if (ac < 0) az = dpi - az
		if (az < 0) az = tpi + az
		ac = -cdec * sha / sin (az)
		alt = atan ((slat * sdec) + (clat * cdec * cha) / abs (ac))
		if (ac < 0) alt = dpi - alt
		az = az / rcon
		alt = alt / rcon
		}
	    else {
		alt = 0.d0
		az = 0.d0
		}

	    exptime = 0.d0
	    call imgdpar (image,"EXPTIME",exptime)

	# Print one-line report
	    do j = 1, nlogfd {
		call fprintf(logfd[j],"%6s %8s %16s")
		    call pargstr (specid)
		    call pargstr (instrument)
		    call pargstr (IM_TITLE(image))
		call fprintf(logfd[j]," %011.2h %010.1h %5.1f %5.1f")
		    call pargd (ra)
		    call pargd (dec)
		    call pargd (az)
		    call pargd (alt)
		call fprintf(logfd[j]," %15.5f %6.1f")
		    call pargd (dj)
		    call pargd (exptime)
		call fprintf(logfd[j]," %8.2f %6.2f %8.2f %6.2f %5.1f")
		    call pargd (spevel)
		    call pargd (speerr)
		    call pargd (spexvel)
		    call pargd (spexerr)
		    call pargd (spexr)
		call fprintf(logfd[j]," %8.2f %6.2f %2d %2d")
		    call pargd (spvel)
		    call pargd (sperr)
		    call pargi (spnl)
		    call pargi (spnlf)
		do i = 1, nref {
		    call fprintf (logfd[j]," %8.2f %6.2f %9.2f %5.2f %6.2f")
			call pargd (velline[i])
			call pargd (errline[i])
			call pargd (htline[i])
			call pargd (widline[i])
			call pargd (eqwline[i])
		    }
		call fprintf(logfd[j], "\n")
		}
	    }
end
# Aug 12 1992	Correctly print z, not 1+z for fit velocities
# Dec  2 1993	Print spectrum number if multispec file

# Apr  8 1993	Set speed of light locally

# Apr 13 1994	Remove unused variables k0, k1 and rlim
# Apr 22 1994	Remove unused variable gwidth
# Apr 25 1994	Declare month as char
# May 17 1994	Add second option for single-line output format
# Jun 15 1994	Add third output option for per-line velocities
# Aug  3 1994	Change common and header from fquot to rvsao
# Aug 10 1994	Fix ut time
# Aug 17 1994	Add weight per line for mode 1 report
# Aug 22 1994	Don't add weight if zero

# Feb  9 1995	Give 2 decimal places for all velocities
# Feb 15 1995	Add heliocentric vel. correction to individual line velocities
# Apr  3 1995	Add fourth output option for single-line per-line velocities
# Apr  5 1995	Add sky coordinates, alt/az and exposure time to 4th option
# Apr 10 1995	Fix alt/az
# May 15 1995	Change all sz_fname to sz_pathname, which is longer
# Jul 13 1995	Get Julian Date from JULDATE instead of JULDAY
# Jul 13 1995	Get UT from JULDATE
# Jul 19 1995	Handle INDEF values for SPVEL and SPXVEL
# Sep 21 1995	Handle INDEF value for SPEVEL

# Jan 15 1997	Add HDJ to JULDATE arguments
# Feb 12 1997	Add modes 5 and 6 to plot emission line template velocity
# May  2 1997	Always test against dindef, not INDEFD
# May  6 1997	Print messages if no position or no observation date
# May 19 1997	Add height and width to mode 4 report
# May 19 1997	Use CORTEMP parameter instead of inline template name
# May 21 1997	Reinitialize all line parameter arrays for each spectrum
# May 22 1997	If CORTEMP is null string, use best template
# Sep 25 1997	Add null print line so this works in Digital Unix
# Sep 25 1997	In mode 1, put file name on separate line
